
/*
 * settings = {
 *     allowDuplicates: boolean
 *     numItems:        integer - number of featured content items to generate.
 *     adSlots:         Array - slots that should be reserved for ads, if enabled
 *     promoSlots:      Array - slots that should be reserved for promos, if enabled
 *     oemSlots:        Array - slots that should be reserved for OEM content
 * }
 */

function RecEngine(settings) {
    this.settings = {
        "allowDuplicates": !!settings.allowDuplicates,
        "numItems": settings.numItems || 6,
        "itemTypes" : settings.itemTypes ? [].concat(settings.itemTypes) : ["musicVideos", "onDemand"],
        "adSlots": settings.adSlots ? [].concat(settings.adSlots) : [],
        "promoSlots": settings.promoSlots ? [].concat(settings.promoSlots) : [],
        "oemSlots": settings.oemSlots ? [].concat(settings.oemSlots) : []
    };
    
    this.ads = false;
    this.promotions = false;
    
    this.recItemContainer = [];
    this.itemCounter = 0;
    this.oemSlotTotal = this.settings.oemSlots.length;
    this.npSlotTotal = this.settings.numItems - this.oemSlotTotal;
}

RecEngine.prototype = {};

RecEngine.prototype.setItemTypes = function recEngineSetItemTypes(itemTypes) {
    this.settings.itemTypes = itemTypes ? [].concat(itemTypes) : ["musicVideos", "onDemand"];
};

RecEngine.prototype.getFeaturedContent = function recEngineGetFeaturedContent() {
    
    this.setSlotTotals(this.npSlotTotal, this.oemSlotTotal);
    for (var i=0; i<this.settings.numItems; i++) {
        
        if (this.settings.oemSlots.indexOf(i) != -1) {
            continue;
        }
        if (this.settings.adSlots.indexOf(i) != -1 && this.ads) {
            // adMan is responsible for ad behavior, so just return an empty RecItem
            this.recItemContainer[i] = new RecItem("ad");
            continue;
        }
        if (this.settings.promoSlots.indexOf(i) != -1 && this.promotions) {
            // Promotional content comes in as a client zone. It's up to the view to bind
            // any behavior associated with it.
            setupClientZones();
            this.recItemContainer[i] = new RecItem("promotion");
            continue;
        }
        
        var recItem; 
        if (this.settings.allowDuplicates) {
            recItem = this.getFeaturedItem(this.settings.itemTypes);
        } else {
            recItem = this.getUniqueFeaturedItem(this.settings.itemTypes);
        }
        this.recItemContainer[i] = recItem;
    }
    
    return this.recItemContainer;
};

RecEngine.prototype.isDuplicate = function recEngineIsDuplicate(recItem) {
    // Don't check for duplicate ads or promotions
    if (recItem.type == "ad" || recItem.type == "promotion") {
        return false;
    }
    
    for (var i=0; i<this.recItemContainer.length; i++) {
        if (!this.recItemContainer[i]) continue;
        if (recItem.type == this.recItemContainer[i].type && recItem.title == this.recItemContainer[i].title) {
            return true;
        }
    }
    return false;
};

RecEngine.prototype.getFeaturedItem = function recEngineGetFeaturedItem(itemTypes) {
    var winner = this.rollDice(itemTypes);
    winner.setBehavior();
    return winner;
};

RecEngine.prototype.getUniqueFeaturedItem = function recEngineGetUniqueFeaturedItem(itemTypes) {
    var recItem, retryCnt = 0;
    do {
        recItem = this.getFeaturedItem(itemTypes);
        retryCnt++;
    } while (this.isDuplicate(recItem) && retryCnt<=10)
    
    return recItem;
};

RecEngine.prototype.rollDice = function recEngineRollDice(itemTypes) {
    for (var i=0; i<itemTypes.length; i++) {
        var obj = mediaXML.featured[itemTypes[i]] || [];
        if (obj.length == 0 && mediaXML[itemTypes[i]]) {
            itemTypes.splice(i,1);
            i--;
        }
    }
    
    if (itemTypes.length == 0) {
        writeLog("debug", "No featured content to display! Using old algorithm to generate content...");
        return this.dontFreakOut();
    }
    var type = itemTypes[Math.round(Math.random()*(itemTypes.length-1))];
    
    var winner = new RecItem(type);
    switch (type) {
        case "onDemand":
            var media = mediaXML.onDemand;
            var theMatrix = mediaXML.featured.onDemand;
            var theChosenOne = theMatrix[Math.round(Math.random()*(theMatrix.length - 1))];
            
            winner.data = media[theChosenOne.channel].items[theChosenOne.show];
            winner.title = winner.data.title;
            winner.thumbnail = theChosenOne.thumbnail;
            break;
            
        case "musicVideos":
            var media = mediaXML.musicVideos;
            var theMatrix = mediaXML.featured.musicVideos;
            var theChosenOne = theMatrix[Math.round(Math.random()*(theMatrix.length - 1))];
            
            winner.data = media[theChosenOne.channel];
            winner.title = winner.data.title;
            winner.thumbnail = theChosenOne.thumbnail;
            break;
            
        case "lifescape":
            // If no va content was found default to LFconnect
            if (typeof vaObj.isAvailable != 'undefined' && vaObj.isAvailable()) {
                var media = eval(vaObj.getPlayListJson());
                var course = Math.round(Math.random()*(media.length - 1));
            
                winner.data = media[course];
                winner.thumbnail = DIR_ROOT+"/www/"+g_mfgStyle+"/images/featured/undefined.png";
                winner.title = IDS_LANGUAGE.featuredContent["featured"];
            }
            break;
            
        case "virtualTrainer":
            winner.title = IDS_LANGUAGE.featuredContent["featuredVt"];
            winner.thumbnail = DIR_ROOT+"/www/"+g_mfgStyle+"/images/featured/virtualTrainer.png";
            break;
            
        case "playlist":
        case "virtualActive":
            // Single level items
            var item = Math.round(Math.random() * (mediaXML[type].length - 1));
            var mediaItem;
            if (item >= 0) {
                mediaItem = mediaXML[type][item];
                winner.data = {
                    'items': [mediaItem],
                    'shuffle': false
                };
            } else {
                break;
            }
            
            if (type == "playlist") {
                winner.title = IDS_LANGUAGE.featuredContent["featuredPlaylist"];
                
                var genres = mediaItem.genres;
                var featuredGenre = genres[Math.round( Math.random()*(genres.length - 1) )].toLowerCase().replace(/&amp;/g, "&").replace(/\s+/g, "").replace(/\//g, "_");
                var featuredImages = [];
                var imageDir = DIR_ROOT+"/www/"+g_mfgStyle+"/images/featured/" + featuredGenre;
                var filelist = npfunctions.getFileList(imageDir, "*");
                $(filelist).find("file").each(function(index, element) {
                    featuredImages.push($(element).text());
                });
                
                winner.thumbnail = imageDir + "/" + featuredImages[Math.round( Math.random()*(featuredImages.length - 1) )];
            } else {
                var theMatrix = mediaXML.featured.virtualActive;
                var theChosenOne = theMatrix[Math.round(Math.random()*(theMatrix.length - 1))];
                
                winner.data = {
                    'items': [mediaXML.virtualActive[theChosenOne.channel]],
                    'shuffle': false
                };
                winner.title = theChosenOne.title;
                winner.thumbnail = theChosenOne.thumbnail;
            }
            break;
    }
    
    return winner;
};

/**
 * If we don't have any featured content display, don't freak out. This function
 * will return a random music channel to display.
 */
RecEngine.prototype.dontFreakOut = function recEngineDontFreakOut() {
    var winner = new RecItem("musicVideos");
    
    var media = mediaXML.musicVideos;
    var genre = Math.round(Math.random()*(media.length - 1));
    var channelName = media[genre].title;
    
    var featuredImages = new Array();
    var imageDir = DIR_ROOT+"/www/"+g_mfgStyle+"/images/featured/" + channelName.toLowerCase().replace(/&amp;/g, "&").replace(/\s+/g, "").replace(/\//g, "_");
    var filelist = npfunctions.getFileList(imageDir, "*");
    $(filelist).find("file").each(function(index, element) {
        featuredImages.push($(element).text());
    });
    
    winner.data = media[genre];
    winner.title = channelName;
    winner.thumbnail = imageDir + "/" + featuredImages[Math.round( Math.random()*(featuredImages.length - 1) )];
    
    if (file_exists(winner.thumbnail) == "0") {
        winner.thumbnail = DIR_ROOT+"/www/"+g_mfgStyle+"/images/featured/undefined.png";
    }
    
    return winner;
};

RecEngine.prototype.enableAds = function recEngineEnableAds(bEnable) { 
    this.ads = (typeof bEnable == "undefined") ? true : !!bEnable; 
};
RecEngine.prototype.enablePromotions = function recEngineEnablePromotions(bEnable) { 
    this.promotions = (typeof bEnable == "undefined") ? true : !!bEnable; 
};

RecEngine.prototype.setSlotTotals = function recEngineSetSlotsTotal(npSlotTotal, oemSlotTotal) {
    this.npSlotTotal = npSlotTotal;
    this.oemSlotTotal = oemSlotTotal;
    recEngineObj.setSlotTotals(this.npSlotTotal, this.oemSlotTotal);
    
    this.recItemContainer = [];
    var j=0, slots = eval(recEngineObj.getSlots());
    for (var i=slots.length-this.oemSlotTotal; i<slots.length; i++) {
        var recItem = new RecItem("button");
        recItem.data = slots[i].data;
        recItem.id = slots[i].id;
        recItem.thumbnail = slots[i].image;
        recItem.title = slots[i].title;
        recItem.setBehavior();
        
        this.recItemContainer[this.settings.oemSlots[j]] = recItem;
        j++;
    }
};

function RecItem(type) {
    this.id = "";
    this.data = {};
    this.title = "";
    this.thumbnail = "";
    this.type = type || "";
    this.weight = 0;
    this.behavior = function() { return false; };
}

RecItem.prototype.setBehavior = function recItemSetBehavior(behavior) {
    var me = this;
    if (this.type == "button") {
        var id = "id=" + this.id;
        if (this.data.length > 0) {
            var data = "data=" + this.data;
            this.behavior = function() { nsSend3("op=button", id, data); return true; };
        }
        else {
            this.behavior = function() { nsSend2("op=button", id); return true; };
        }
    }
    else {
        if (typeof behavior == "function") {
            this.behavior = behavior;
        } else {
            this.behavior = function() {
                // Play the featured content
                var watchEpisode = me.data.episode || 0;
                
                watchObj.target = {
                    "parentView" : viewManager.currentView,
                    "channel" : me.data,
                    "episode" : watchEpisode
                };
                
                viewManager.show("watch");
                return true;
            };
        }
    }
};
